#==============================================================================
# ★RGSS2 
# STR33b_ATBカスタマイズセクション v1.2 08/05/21
#
# ・STR33の設定プラグインです。
# 　設定箇所の下に解説があります。
#
# ※※注意※※
#　◆"変数"を扱う設定項目があります。
#　よくこれが原因でフリーズしてしまう、などといった症状が
#　報告されていますのでご注意ください。
#
#------------------------------------------------------------------------------
#
# 更新履歴
# ◇1.2→1.3
#　細かい部分を修正(EXTEND:updateのデフォ値を2に、スピード取得の仕様変更)
# ◇1.1→1.2
#　本体ver1.0以降対応 設定項目いろいろ追加、マニュアル追記
# ◇1.0→1.1
#　本体ver8.0専用 設定項目＋マニュアル追加
# ◇1.0
#　暫定正式バージョン
#
#==============================================================================
# ■ STRRGSS2
#==============================================================================
module STRRGSS2;module CTB
  #-------------------------------------------------------------------------#
  # ★ 変数依存設定                                                         #
  #-------------------------------------------------------------------------#
  # ◆ウェイトモードを指定した変数に依存させる
  WAIT_VA = 0         # する場合、1-5000の変数IDを指定　しない場合は 0
  # ◆ゲージスピードを指定した変数に依存させる
  CT_SPEED_VA = 0     # する場合、1-5000の変数IDを指定　しない場合は 0
  #-------------------------------------------------------------------------#
  # ★ ウェイト                                                             #
  #-------------------------------------------------------------------------#
  # ウェイトモード
  WAIT   = 0          # ★パターンを参照↓
  # バトルウェイトパターン ※なるべく変更しないでください※
  #            A     B     C     D     # パターン値
  WAIT_P = [[false,false,false,false], # 0:フルアクティブ
            [false, true,false,false], # 1:FFウェイトモード
            [false, true, true,false], # 2:1+ターゲット選択中もウェイト
            [false, true, true, true], # 3:コマンド入力中はウェイト
            [ true, true, true, true]] # 4:完全ウェイト
                      # A = アクション実行中
                      # B = アイテム・スキル選択中
                      # C = ターゲット選択中
                      # D = アクターコマンド選択中(攻撃スキル防御アイテム)
  #-------------------------------------------------------------------------#
  # ★ ターン経過タイミング(v1.0)                                           #
  #-------------------------------------------------------------------------#
  RTT       = true  # ターン経過を時間経過式に変更する
                    # true ：有効(指定した時間が経過すると 1 ターン経過)　
                    # false：無効(ユニットが行動する毎に 1 ターン経過)
  RTT_COUNT = 300   # 1ターン経過とみなす時間(1/60sec、RTTがtrueの時のみ有効)
  # 参考)エネミーの行動条件の"ターン"は別に管理されます(詳細は仕様を参照)
  #-------------------------------------------------------------------------#
  # ★ カウントゲージ                                                       #
  #-------------------------------------------------------------------------#
  CT_MAX       = 256        # カウント最大値
  CT_SPEED     = 2          # カウントスピード
  CT_CANCEL    = 20         # キャンセル時(行動送り)に消費するカウント(％)
  # アクターコマンドオープン時のSE
  COMMANDSE    = RPG::SE.new("Decision2", 80, 100) 
  # 通常攻撃によるカウントダメージ(％)
  CT_DM_ATTACK = 0          # -100～100の間で指定
  # アイテム・スキルのカウントダメージ設定ワード
  CT_DM_WORD   = "カウントダメージ"
  # 武器・防具・ステートのスピードレート設定ワード(v1.0)
  CT_CT_WORD   = "CTスピードレート"
  CT_AW_WORD   = "詠唱スピードレート"
  #-------------------------------------------------------------------------#
  # ★ 逃走設定                                                             #
  #-------------------------------------------------------------------------#
  ESCAPE       = 192        # 逃げやすさ(低いほど逃げやすくなる)
  ESCAPE_KEY   = Input::L   # 逃走ボタン
  ESCAPE_SWITCH= false      # 押しっぱなし式からスイッチ式に変更する(v1.0)
  ESCAPE_CSTOP = true       # 逃走中はアクターのカウントを止める
  ESCAPE_LOSS  = true       # 逃走中で無い時に逃走カウントを減少させる
  ESCAPE_LOSSP = 768        # ・カウントを減らす速さ(低いほど早い)
  #-------------------------------------------------------------------------#
  # ★ レイアウト                                                           #
  #-------------------------------------------------------------------------#
  W_RESIZE     = false      # レイアウト変更の設定を有効にする
  W_HELP_Y     = 232        # スキル/アイテムのヘルプウィンドウY座標
  # スキル/アイテムウィンドウ矩形
  W_RECT       = Rect.new(0, 288, 544, 128)
  #-------------------------------------------------------------------------#
  # ★ 機能無効化設定(v1.0)                                                 #
  #-------------------------------------------------------------------------#
  ESC_DISABLE = false # 逃走を無効にする
  CNX_DISABLE = false # アクターコマンドキャンセル(行動送り)を無効にする
  #-------------------------------------------------------------------------#
  # ★ 拡張設定(通常は変更しないでください)                                 #
  #-------------------------------------------------------------------------#
  EXTEND       = {:update => 1}
  #-------------------------------------------------------------------------#
  # ★ 非設定箇所                                                           #
  #-------------------------------------------------------------------------#
  # ウェイトモード取得
  def self.wait_mode
    if WAIT_VA == 0
      return WAIT_P[WAIT]
    else
      return WAIT_P[$game_variables[WAIT_VA]]
    end
  end
  # ゲージスピード取得
  def self.ctspeed
    if CT_SPEED_VA == 0
      return CT_SPEED * EXTEND[:update]
    else
      return $game_variables[CT_SPEED_VA] * EXTEND[:update]
    end
  end
  # !!戦闘テストの場合、変数依存設定を無効にする!!
  WAIT_VA = CT_SPEED_VA = 0 if $BTEST
  #
end;end
#==============================================================================
# ■ Scene_Battle
#==============================================================================
class Scene_Battle < Scene_Base
  #--------------------------------------------------------------------------
  # ● スキル選択の開始(再定義)
  #--------------------------------------------------------------------------
  if STRRGSS2::CTB::W_RESIZE;def start_skill_selection_str33x
    @help_window = Window_Help.new
    @help_window.y = STRRGSS2::CTB::W_HELP_Y
    r = STRRGSS2::CTB::W_RECT
    @skill_window = Window_Skill.new(r.x, r.y, r.width, r.height, @active_battler)
    @skill_window.help_window = @help_window
    @actor_command_window.active = false
  end
  #--------------------------------------------------------------------------
  # ● アイテム選択の開始(再定義)
  #--------------------------------------------------------------------------
  def start_item_selection
    @help_window = Window_Help.new
    @help_window.y = STRRGSS2::CTB::W_HELP_Y
    r = STRRGSS2::CTB::W_RECT
    @item_window = Window_Item.new(r.x, r.y, r.width, r.height)
    @item_window.help_window = @help_window
    @actor_command_window.active = false
  end;end
end
#==============================================================================
# ★ STR33_アクティブタイムバトル マニュアル
#------------------------------------------------------------------------------
# ◆概要
#　戦闘に時間の概念を加え、FFのATBのような戦闘形式にします。
#　敏捷性の高いバトラーほど早くカウントが溜まり、設定箇所で指定した最大値まで
#　溜まると行動出来るようになります。
#　行動したバトラーのカウントは 0 になり、再びカウントが最大まで溜まるまで
#　行動することが出来なくなります。
#------------------------------------------------------------------------------
# ◆主な機能
#　□押しっぱなし逃走
#　　このATBスクリプトではパーティコマンドでの逃走が不可になります。
#　　その代わり、設定箇所で指定したボタンを押しっぱなしにすることで
#　　時間経過で逃走することが出来るようになります。
#　　逃走不可のバトルでは勿論逃走を行うことは出来ません。
#　□アイテム/スキル詠唱
#　　データベース標準の"速度補正値"の値を利用して、
#　　アイテム/スキルの発動を遅延させることが出来ます。
#　　スピード計算式は下の仕様の項目をご覧ください。
#　□カウントダメージ
#　　アイテム/スキルのメモ欄を利用することで、
#　　対象に追加効果としてCTを%増減させるアイテム/スキルを作成することが出来ます。
#　　・メモへの記述方法
#　　 カウントダメージ[10]
#　　　　　　　　　　　↑ここに -100 ～ 100 の値を指定します。
#　　　　　　　　　　　　指定した値の％のCTダメージを与えます(マイナスで回復)
#　□カウント/詠唱スピードレート変動装備・ステート
#　　武器/防具/ステートのメモ欄を利用することで、カウント/詠唱スピードを
#　　％単位で変動させる装備/ステートを作成することが出来ます。
#　　メモへの記述方法
#　　　・カウントスピードレートの場合
#　　 　CTスピードレート[200%]
#　　　　　　　　　　　　↑ここに 0 以上の値を指定します。
#　　　・詠唱スピードレートの場合
#　　 　詠唱スピードレート[150%]
#　　　　　　　　　　　　　↑ここに 0 以上の値を指定します。
#------------------------------------------------------------------------------
# ◆仕様
#　□カウント関連
#　・カウントスピード
#　　AGI / 全バトラー平均AGI * CT_SPEED
#　　※ AGI = 敏捷性
#　　※ 平均AGIはターンごとに算出される
#　・詠唱スピード
#　　カウントスピード * 速度補正値 / 100
#　　　例) 速度補正値 150 でカウントスピードの1.5倍のカウントに
#　・初期カウント値
#　　通常　　：最大CT  33% ～ 最大CT  43%
#　　武器先制：最大CT 100%
#　　先制　　：最大CT  90% ～ 最大CT 100%
#　　不意打ち：0
#　　　※ 武器先制 = 武器オプションの"ターン内先制"。このATBではターン内先制の
#　　　　 代替の効果として、バトル開始と同時にカウント100%になります。
#　　<補足> 不意打ちと武器先制が重なった場合、敵先制の方が優先されます
#　・カウントダメージ
#　　カウントが最大値に達したユニットへのCTダメージは無効です
#　・CT/詠唱スピードレート
#　　1.0 * (全装備、全付加ステートに設定されている値 / 100 の合計の積)
#　　　例) 武器150% 防具80% ステート200% で 240%
#　　　例) 武器120% 防具200% ステート0% で 0%
#　　
#　□逃走関連
#　・逃走カウントスピード
#　　パーティ平均の敏捷性
#　・逃走カウント最大値
#　　(敵グループ平均の敏捷性 * 敵の数 * 設定項目の ESCAPE の値) / 2
#　　+ (0 ～ 敵グループ平均の敏捷性 * 2)
#　・逃走カウント減少スピード
#　　パーティ平均の敏捷性 / 設定項目の ESCAPE_LOSSP の値
#　・初期逃走カウント値
#　　通常　　：最大ESC  0% ～ 最大ESC  20%
#　　先制　　：最大ESC 50% ～ 最大ESC 100%
#　　不意打　：0
#　　※ ESC = 逃走カウント
#
#　□挙動
#　・通常メッセージは表示されている間はカウントが停止します。
#　・行動不可になるステートに掛かるとCT値は0になります。
#　・敵キャラの行動条件の"ターン"は、バトル全体のターンではなく、
#　　その敵キャラの行動回数に置き換えて判定します。
#　・パーティコマンドは無効になります。
#
#　・[v0.8]誰かが行動するたびに 1 ターン加算するようになります。
#　　バトルイベントの条件に"ターン"を用いる場合と
#　　ステートの解除条件のターン経過の設定で、
#　　この仕様を考慮しなければなりません。
#　　※この仕様により、スリップダメージが凶悪な強さになるので
#　　　他の素材サイトにあるスリップダメージ調整の
#　　　スクリプトを導入することをお勧めします。
#　　☆v1.0でターン経過タイミングを時間経過式に設定できるようになりました。
#
#==============================================================================